@@ -1,5 +1,5 @@
 /*
- * $Id: scp_write.c,v 1.6 2008/11/10 16:48:41 bagder Exp $
+ * $Id: scp_write.c,v 1.7 2009/04/28 10:35:30 bagder Exp $
  *
  * Sample showing how to do a simple SCP transfer.
  */
@@ -10,12 +10,12 @@
 #ifdef HAVE_WINSOCK2_H
 # include <winsock2.h>
 #endif
-#ifdef HAVE_NETINET_IN_H
-# include <netinet/in.h>
-#endif
 #ifdef HAVE_SYS_SOCKET_H
 # include <sys/socket.h>
 #endif
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
 # ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -47,7 +47,7 @@ int main(int argc, char *argv[])
     FILE *local;
     int rc;
     char mem[1024];
-    size_t nread;
+    size_t nread, sent;
     char *ptr;
     struct stat fileinfo;
 
@@ -142,8 +142,8 @@ int main(int argc, char *argv[])
         }
     }
 
-    /* Request a file via SCP */
-    channel = libssh2_scp_send(session, scppath, 0x1FF & fileinfo.st_mode,
+    /* Send a file via scp. The mode parameter must only have permissions! */
+    channel = libssh2_scp_send(session, scppath, fileinfo.st_mode & 0777,
                                (unsigned long)fileinfo.st_size);
 
     if (!channel) {
@@ -159,13 +159,20 @@ int main(int argc, char *argv[])
             break;
         }
         ptr = mem;
+        sent = 0;
 
         do {
-            /* write data in a loop until we block */
+            /* write the same data over and over, until error or completion */
             rc = libssh2_channel_write(channel, ptr, nread);
-            ptr += rc;
-            nread -= nread;
-        } while (rc > 0);
+            if (rc < 0) {
+                fprintf(stderr, "ERROR %d\n", rc);
+            } else {
+                /* rc indicates how many bytes were written this time */
+                sent += rc;
+            }
+        } while (rc > 0 && sent < nread);
+        ptr += sent;
+        nread -= sent;
     } while (1);
 
     fprintf(stderr, "Sending EOF\n");
@@ -186,10 +193,8 @@ int main(int argc, char *argv[])
     libssh2_session_free(session);
 
 #ifdef WIN32
-    Sleep(1000);
     closesocket(sock);
 #else
-    sleep(1);
     close(sock);
 #endif
     fprintf(stderr, "all done\n");
