@@ -1,5 +1,5 @@
 /*
- * $Id: sftp_mkdir.c,v 1.8 2008/11/10 16:48:41 bagder Exp $
+ * $Id: sftp_mkdir.c,v 1.9 2009/04/28 10:35:30 bagder Exp $
  *
  * Sample showing how to do SFTP mkdir
  *
@@ -16,12 +16,12 @@
 #ifdef HAVE_WINSOCK2_H
 # include <winsock2.h>
 #endif
-#ifdef HAVE_NETINET_IN_H
-# include <netinet/in.h>
-#endif
 #ifdef HAVE_SYS_SOCKET_H
 # include <sys/socket.h>
 #endif
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
 # ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -155,10 +155,8 @@ int main(int argc, char *argv[])
     libssh2_session_free(session);
 
 #ifdef WIN32
-    Sleep(1000);
     closesocket(sock);
 #else
-    sleep(1);
     close(sock);
 #endif
 printf("all done\n");
