@@ -1,5 +1,5 @@
 /*
- * $Id: sftp_write.c,v 1.10 2009/03/31 12:20:36 bagder Exp $
+ * $Id: sftp_write.c,v 1.11 2009/04/28 10:35:30 bagder Exp $
  *
  * Sample showing how to do SFTP write transfers.
  *
@@ -16,12 +16,12 @@
 #ifdef HAVE_WINSOCK2_H
 # include <winsock2.h>
 #endif
-#ifdef HAVE_NETINET_IN_H
-# include <netinet/in.h>
-#endif
 #ifdef HAVE_SYS_SOCKET_H
 # include <sys/socket.h>
 #endif
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
 # ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -195,10 +195,8 @@ int main(int argc, char *argv[])
     libssh2_session_free(session);
 
 #ifdef WIN32
-    Sleep(1000);
     closesocket(sock);
 #else
-    sleep(1);
     close(sock);
 #endif
 printf("all done\n");
