@@ -93,17 +93,17 @@ static int sftp_close_handle(LIBSSH2_SFTP_HANDLE *handle);
 static void
 _libssh2_htonu64(unsigned char *buf, libssh2_uint64_t value)
 {
-    unsigned long msl = ((libssh2_uint64_t)value >> 32);
+    unsigned long msl = (unsigned long)(value >> 32);
 
-    buf[0] = (msl >> 24) & 0xFF;
-    buf[1] = (msl >> 16) & 0xFF;
-    buf[2] = (msl >> 8) & 0xFF;
-    buf[3] = msl & 0xFF;
+    buf[0] = (unsigned char)((msl >> 24) & 0xFF);
+    buf[1] = (unsigned char)((msl >> 16) & 0xFF);
+    buf[2] = (unsigned char)((msl >> 8)  & 0xFF);
+    buf[3] = (unsigned char)( msl        & 0xFF);
 
-    buf[4] = (value >> 24) & 0xFF;
-    buf[5] = (value >> 16) & 0xFF;
-    buf[6] = (value >> 8) & 0xFF;
-    buf[7] = value & 0xFF;
+    buf[4] = (unsigned char)((value >> 24) & 0xFF);
+    buf[5] = (unsigned char)((value >> 16) & 0xFF);
+    buf[6] = (unsigned char)((value >> 8)  & 0xFF);
+    buf[7] = (unsigned char)( value        & 0xFF);
 }
 
 /*
@@ -124,22 +124,15 @@ sftp_packet_add(LIBSSH2_SFTP *sftp, unsigned char *data,
     if (!packet) {
         libssh2_error(session, LIBSSH2_ERROR_ALLOC,
                       "Unable to allocate datablock for SFTP packet", 0);
-        return -1;
+        return LIBSSH2_ERROR_ALLOC;
     }
     memset(packet, 0, sizeof(LIBSSH2_PACKET));
 
     packet->data = data;
     packet->data_len = data_len;
     packet->data_head = 5;
-    packet->brigade = &sftp->packets;
-    packet->next = NULL;
-    packet->prev = sftp->packets.tail;
-    if (packet->prev) {
-        packet->prev->next = packet;
-    } else {
-        sftp->packets.head = packet;
-    }
-    sftp->packets.tail = packet;
+
+    _libssh2_list_add(&sftp->packets, &packet->node);
 
     return 0;
 }
@@ -174,17 +167,17 @@ sftp_packet_read(LIBSSH2_SFTP *sftp)
                        "partial read cont, len: %lu", packet_len);
     }
     else {
-        rc = libssh2_channel_read_ex(channel, 0, (char *) buffer, 4);
+        rc = _libssh2_channel_read(channel, 0, (char *) buffer, 4);
         if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return rc;
         }
         else if (4 != rc) {
             /* TODO: this is stupid since we can in fact get 1-3 bytes in a
                legitimate working case as well if the connection happens to be
                super slow or something */
-            libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
-                          "Timeout waiting for FXP packet", 0);
-            return -1;
+            libssh2_error(session, LIBSSH2_ERROR_CHANNEL_FAILURE,
+                          "Read part of packet", 0);
+            return LIBSSH2_ERROR_CHANNEL_FAILURE;
         }
 
         packet_len = _libssh2_ntohu32(buffer);
@@ -193,14 +186,14 @@ sftp_packet_read(LIBSSH2_SFTP *sftp)
         if (packet_len > LIBSSH2_SFTP_PACKET_MAXLEN) {
             libssh2_error(session, LIBSSH2_ERROR_CHANNEL_PACKET_EXCEEDED,
                           "SFTP packet too large", 0);
-            return -1;
+            return LIBSSH2_ERROR_CHANNEL_PACKET_EXCEEDED;
         }
 
         packet = LIBSSH2_ALLOC(session, packet_len);
         if (!packet) {
             libssh2_error(session, LIBSSH2_ERROR_ALLOC,
                           "Unable to allocate SFTP packet", 0);
-            return -1;
+            return LIBSSH2_ERROR_ALLOC;
         }
 
         packet_received = 0;
@@ -209,9 +202,9 @@ sftp_packet_read(LIBSSH2_SFTP *sftp)
     /* Read as much of the packet as we can */
     while (packet_len > packet_received) {
         bytes_received =
-            libssh2_channel_read_ex(channel, 0,
-                                    (char *) packet + packet_received,
-                                    packet_len - packet_received);
+            _libssh2_channel_read(channel, 0,
+                                  (char *) packet + packet_received,
+                                  packet_len - packet_received);
 
         if (bytes_received == PACKET_EAGAIN) {
             /*
@@ -223,20 +216,21 @@ sftp_packet_read(LIBSSH2_SFTP *sftp)
             sftp->partial_received = packet_received;
             packet = NULL;
 
-            return PACKET_EAGAIN;
+            return bytes_received;
         }
         else if (bytes_received < 0) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
                           "Receive error waiting for SFTP packet", 0);
             LIBSSH2_FREE(session, packet);
-            return -1;
+            return bytes_received;
         }
         packet_received += bytes_received;
     }
 
-    if (sftp_packet_add(sftp, packet, packet_len)) {
+    rc = sftp_packet_add(sftp, packet, packet_len);
+    if (rc) {
         LIBSSH2_FREE(session, packet);
-        return -1;
+        return rc;
     }
 
     return packet[0];
@@ -253,7 +247,7 @@ sftp_packet_ask(LIBSSH2_SFTP *sftp, unsigned char packet_type,
                 unsigned long *data_len)
 {
     LIBSSH2_SESSION *session = sftp->channel->session;
-    LIBSSH2_PACKET *packet = sftp->packets.head;
+    LIBSSH2_PACKET *packet = _libssh2_list_first(&sftp->packets);
     unsigned char match_buf[5];
     int match_len;
 
@@ -277,24 +271,13 @@ sftp_packet_ask(LIBSSH2_SFTP *sftp, unsigned char packet_type,
             *data_len = packet->data_len;
 
             /* unlink and free this struct */
-            if (packet->prev) {
-                packet->prev->next = packet->next;
-            } else {
-                sftp->packets.head = packet->next;
-            }
-
-            if (packet->next) {
-                packet->next->prev = packet->prev;
-            } else {
-                sftp->packets.tail = packet->prev;
-            }
-
+            _libssh2_list_remove(&packet->node);
             LIBSSH2_FREE(session, packet);
 
             return 0;
         }
         /* check next struct in the list */
-        packet = packet->next;
+        packet = _libssh2_list_next(&packet->node);
     }
     return -1;
 }
@@ -323,7 +306,7 @@ sftp_packet_require(LIBSSH2_SFTP *sftp, unsigned char packet_type,
     while (session->socket_state == LIBSSH2_SOCKET_CONNECTED) {
         ret = sftp_packet_read(sftp);
         if (ret == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return ret;
         } else if (ret <= 0) {
             return -1;
         }
@@ -338,7 +321,7 @@ sftp_packet_require(LIBSSH2_SFTP *sftp, unsigned char packet_type,
     }
 
     /* Only reached if the socket died */
-    return -1;
+    return LIBSSH2_ERROR_SOCKET_DISCONNECT;
 }
 
 /* sftp_packet_requirev
@@ -378,14 +361,14 @@ sftp_packet_requirev(LIBSSH2_SFTP *sftp, int num_valid_responses,
         } else if (ret <= 0) {
             /* prevent busy-looping */
             long left =
-                LIBSSH2_READ_TIMEOUT - (time(NULL) - sftp->requirev_start);
+                LIBSSH2_READ_TIMEOUT - (long)(time(NULL) - sftp->requirev_start);
 
             if (left <= 0) {
                 sftp->requirev_start = 0;
                 return PACKET_TIMEOUT;
             }
             else if (ret == PACKET_EAGAIN) {
-                return PACKET_EAGAIN;
+                return ret;
             }
         }
     }
@@ -521,20 +504,6 @@ LIBSSH2_CHANNEL_CLOSE_FUNC(libssh2_sftp_dtor)
     (void) session_abstract;
     (void) channel;
 
-#if 0
-    /* EEEK! While it might sound like a neat idea to make this code loop over
-       all the outstanding handles and close them, that is going to cause
-       EAGAIN to get returned and this callback system is not designed to
-       handle this very nicely so thus we now DEMAND that the app closes its
-       handles instead!
-    */
-
-    /* Loop through handles closing them */
-    while (sftp->handles) {
-        sftp_close_handle(sftp->handles);
-    }
-#endif
-
     /* Free the partial packet storage for sftp_packet_read */
     if (sftp->partial_packet) {
         LIBSSH2_FREE(session, sftp->partial_packet);
@@ -558,18 +527,31 @@ static LIBSSH2_SFTP *sftp_init(LIBSSH2_SESSION *session)
     unsigned char *data, *s;
     unsigned long data_len;
     int rc;
+    LIBSSH2_SFTP *sftp_handle;
 
     if (session->sftpInit_state == libssh2_NB_state_idle) {
         _libssh2_debug(session, LIBSSH2_DBG_SFTP,
                        "Initializing SFTP subsystem");
 
+        /*
+         * The 'sftpInit_sftp' and 'sftpInit_channel' struct fields within the
+         * session struct are only to be used during the setup phase. As soon
+         * as the SFTP session is created they are cleared and can thus be
+         * re-used again to allow any amount of SFTP handles per sessions.
+         *
+         * Note that you MUST NOT try to call libssh2_sftp_init() again to get
+         * another handle until the previous call has finished and either
+         * succesffully made a handle or failed and returned error (not
+         * including *EAGAIN).
+         */
+
         assert(session->sftpInit_sftp == NULL);
-
         session->sftpInit_sftp = NULL;
-
         session->sftpInit_state = libssh2_NB_state_created;
     }
 
+    sftp_handle = session->sftpInit_sftp;
+
     if (session->sftpInit_state == libssh2_NB_state_created) {
         session->sftpInit_channel =
             _libssh2_channel_open(session, "session", sizeof("session") - 1,
@@ -618,15 +600,17 @@ static LIBSSH2_SFTP *sftp_init(LIBSSH2_SESSION *session)
             return NULL;
         }
 
-        session->sftpInit_sftp = LIBSSH2_ALLOC(session, sizeof(LIBSSH2_SFTP));
-        if (!session->sftpInit_sftp) {
+        sftp_handle =
+            session->sftpInit_sftp =
+            LIBSSH2_ALLOC(session, sizeof(LIBSSH2_SFTP));
+        if (!sftp_handle) {
             libssh2_error(session, LIBSSH2_ERROR_ALLOC,
                           "Unable to allocate a new SFTP structure", 0);
             goto sftp_init_error;
         }
-        memset(session->sftpInit_sftp, 0, sizeof(LIBSSH2_SFTP));
-        session->sftpInit_sftp->channel = session->sftpInit_channel;
-        session->sftpInit_sftp->request_id = 0;
+        memset(sftp_handle, 0, sizeof(LIBSSH2_SFTP));
+        sftp_handle->channel = session->sftpInit_channel;
+        sftp_handle->request_id = 0;
 
         _libssh2_htonu32(session->sftpInit_buffer, 5);
         session->sftpInit_buffer[4] = SSH_FXP_INIT;
@@ -655,7 +639,7 @@ static LIBSSH2_SFTP *sftp_init(LIBSSH2_SESSION *session)
         session->sftpInit_state = libssh2_NB_state_sent3;
     }
 
-    rc = sftp_packet_require(session->sftpInit_sftp, SSH_FXP_VERSION,
+    rc = sftp_packet_require(sftp_handle, SSH_FXP_VERSION,
                              0, &data, &data_len);
     if (rc == PACKET_EAGAIN) {
         libssh2_error(session, LIBSSH2_ERROR_EAGAIN,
@@ -674,17 +658,17 @@ static LIBSSH2_SFTP *sftp_init(LIBSSH2_SESSION *session)
     }
 
     s = data + 1;
-    session->sftpInit_sftp->version = _libssh2_ntohu32(s);
+    sftp_handle->version = _libssh2_ntohu32(s);
     s += 4;
-    if (session->sftpInit_sftp->version > LIBSSH2_SFTP_VERSION) {
+    if (sftp_handle->version > LIBSSH2_SFTP_VERSION) {
         _libssh2_debug(session, LIBSSH2_DBG_SFTP,
                        "Truncating remote SFTP version from %lu",
-                       session->sftpInit_sftp->version);
-        session->sftpInit_sftp->version = LIBSSH2_SFTP_VERSION;
+                       sftp_handle->version);
+        sftp_handle->version = LIBSSH2_SFTP_VERSION;
     }
     _libssh2_debug(session, LIBSSH2_DBG_SFTP,
                    "Enabling SFTP version %lu compatability",
-                   session->sftpInit_sftp->version);
+                   sftp_handle->version);
     while (s < (data + data_len)) {
         unsigned char *extension_name, *extension_data;
         unsigned long extname_len, extdata_len;
@@ -705,11 +689,18 @@ static LIBSSH2_SFTP *sftp_init(LIBSSH2_SESSION *session)
 
     /* Make sure that when the channel gets closed, the SFTP service is shut
        down too */
-    session->sftpInit_sftp->channel->abstract = session->sftpInit_sftp;
-    session->sftpInit_sftp->channel->close_cb = libssh2_sftp_dtor;
+    sftp_handle->channel->abstract = sftp_handle;
+    sftp_handle->channel->close_cb = libssh2_sftp_dtor;
 
     session->sftpInit_state = libssh2_NB_state_idle;
-    return session->sftpInit_sftp;
+
+    /* clear the sftp and channel pointers in this session struct now */
+    session->sftpInit_sftp = NULL;
+    session->sftpInit_channel = NULL;
+
+    _libssh2_list_init(&sftp_handle->sftp_handles);
+
+    return sftp_handle;
 
   sftp_init_error:
     while (_libssh2_channel_free(session->sftpInit_channel) == PACKET_EAGAIN);
@@ -792,6 +783,9 @@ sftp_shutdown(LIBSSH2_SFTP *sftp)
         sftp->symlink_packet = NULL;
     }
 
+    /* TODO: We should consider walking over the sftp_handles list and kill
+     * any remaining sftp handles ... */
+
     rc = _libssh2_channel_free(sftp->channel);
 
     return rc;
@@ -804,8 +798,7 @@ LIBSSH2_API int
 libssh2_sftp_shutdown(LIBSSH2_SFTP *sftp)
 {
     int rc;
-    BLOCK_ADJUST(rc, sftp->channel->session,
-                 sftp_shutdown(sftp));
+    BLOCK_ADJUST(rc, sftp->channel->session, sftp_shutdown(sftp));
     return rc;
 }
 
@@ -914,7 +907,7 @@ sftp_open(LIBSSH2_SFTP *sftp, const char *filename,
             return NULL;
         }
         else if (rc) {
-            libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
+            libssh2_error(session, rc,
                           "Timeout waiting for status message", 0);
             sftp->open_state = libssh2_NB_state_idle;
             return NULL;
@@ -979,12 +972,10 @@ sftp_open(LIBSSH2_SFTP *sftp, const char *filename,
     memcpy(fp->handle, data + 9, fp->handle_len);
     LIBSSH2_FREE(session, data);
 
-    /* Link the file and the sftp session together */
-    fp->next = sftp->handles;
-    if (fp->next) {
-        fp->next->prev = fp;
-    }
-    fp->sftp = sftp;
+    /* add this file handle to the list kept in the sftp session */
+    _libssh2_list_add(&sftp->sftp_handles, &fp->node);
+
+    fp->sftp = sftp; /* point to the parent struct */
 
     fp->u.file.offset = 0;
 
@@ -1100,12 +1091,12 @@ static ssize_t sftp_read(LIBSSH2_SFTP_HANDLE * handle, char *buffer,
                 sftp->read_packet = packet;
                 sftp->read_request_id = request_id;
                 sftp->read_total_read = total_read;
-                return PACKET_EAGAIN;
+                return retcode;
             } else if (packet_len != retcode) {
                 /* TODO: a partial write is not a critical error when in
                    non-blocking mode! */
                 libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
-                              "Unable to send FXP_READ command", 0);
+                              "_libssh2_channel_write() failed", 0);
                 sftp->read_packet = NULL;
                 sftp->read_state = libssh2_NB_state_idle;
                 return -1;
@@ -1121,9 +1112,11 @@ static ssize_t sftp_read(LIBSSH2_SFTP_HANDLE * handle, char *buffer,
                 sftp_packet_requirev(sftp, 2, read_responses,
                                      request_id, &data, &data_len);
             if (retcode == PACKET_EAGAIN) {
-                return PACKET_EAGAIN;
+                libssh2_error(session, retcode,
+                              "Would block waiting for status message", 0);
+                return retcode;
             } else if (retcode) {
-                libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
+                libssh2_error(session, retcode,
                               "Timeout waiting for status message", 0);
                 sftp->read_packet = NULL;
                 sftp->read_state = libssh2_NB_state_idle;
@@ -1301,11 +1294,11 @@ static int sftp_readdir(LIBSSH2_SFTP_HANDLE *handle, char *buffer,
                                          (char *) sftp->readdir_packet,
                                          packet_len);
         if (retcode == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return retcode;
         }
         else if (packet_len != retcode) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
-                          "Unable to send FXP_READ command", 0);
+                          "_libssh2_channel_write() failed", 0);
             LIBSSH2_FREE(session, sftp->readdir_packet);
             sftp->readdir_packet = NULL;
             sftp->readdir_state = libssh2_NB_state_idle;
@@ -1323,7 +1316,7 @@ static int sftp_readdir(LIBSSH2_SFTP_HANDLE *handle, char *buffer,
                              sftp->readdir_request_id, &data,
                              &data_len);
     if (retcode == PACKET_EAGAIN) {
-        return PACKET_EAGAIN;
+        return retcode;
     } else if (retcode) {
         libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
                       "Timeout waiting for status message", 0);
@@ -1409,10 +1402,13 @@ libssh2_sftp_readdir_ex(LIBSSH2_SFTP_HANDLE *hnd, char *buffer,
     return rc;
 }
 
-/* sftp_write
- * Write data to a file handle
+/*
+ * sftp_write
+ *
+ * Write data to an SFTP handle. Returns the number of bytes written, or
+ * a negative error code.
  */
-static ssize_t sftp_write(LIBSSH2_SFTP_HANDLE * handle, const char *buffer,
+static ssize_t sftp_write(LIBSSH2_SFTP_HANDLE *handle, const char *buffer,
                           size_t count)
 {
     LIBSSH2_SFTP *sftp = handle->sftp;
@@ -1421,20 +1417,28 @@ static ssize_t sftp_write(LIBSSH2_SFTP_HANDLE * handle, const char *buffer,
     unsigned long data_len, retcode;
     /* 25 = packet_len(4) + packet_type(1) + request_id(4) + handle_len(4) +
        offset(8) + count(4) */
-    ssize_t packet_len = handle->handle_len + count + 25;
+    ssize_t packet_len;
     unsigned char *s, *data;
     int rc;
 
+    /* There's no point in us accepting a VERY large packet here since we
+       cannot send it anyway. We just accept 4 times the big size to fill up
+       the queue somewhat. */
+
+    if(count > (MAX_SSH_PACKET_LEN*4))
+        count = MAX_SSH_PACKET_LEN*4;
+
+    packet_len = handle->handle_len + count + 25;
+
     if (sftp->write_state == libssh2_NB_state_idle) {
         _libssh2_debug(session, LIBSSH2_DBG_SFTP, "Writing %lu bytes",
                        (unsigned long) count);
         s = sftp->write_packet = LIBSSH2_ALLOC(session, packet_len);
         if (!sftp->write_packet) {
             libssh2_error(session, LIBSSH2_ERROR_ALLOC,
-                          "Unable to allocate memory for FXP_WRITE packet", 0);
-            return -1;
+                          "Unable to allocate memory for FXP_WRITE", 0);
+            return LIBSSH2_ERROR_ALLOC;
         }
-
         _libssh2_htonu32(s, packet_len - 4);
         s += 4;
         *(s++) = SSH_FXP_WRITE;
@@ -1456,18 +1460,18 @@ static ssize_t sftp_write(LIBSSH2_SFTP_HANDLE * handle, const char *buffer,
     }
 
     if (sftp->write_state == libssh2_NB_state_created) {
-        rc = _libssh2_channel_write(channel, 0, (char *) sftp->write_packet,
+        rc = _libssh2_channel_write(channel, 0, (char *)sftp->write_packet,
                                     packet_len);
-        if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+        if(rc < 0) {
+            /* error */
+            return rc;
         }
-        if (packet_len != rc) {
-            libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
-                          "Unable to send FXP_READ command", 0);
-            LIBSSH2_FREE(session, sftp->write_packet);
-            sftp->write_packet = NULL;
-            sftp->write_state = libssh2_NB_state_idle;
-            return -1;
+        else if(0 == rc) {
+            /* nothing sent is an error */
+            return LIBSSH2_ERROR_SOCKET_SEND;
+        }
+        else if (packet_len != rc) {
+            return rc;
         }
         LIBSSH2_FREE(session, sftp->write_packet);
         sftp->write_packet = NULL;
@@ -1477,12 +1481,13 @@ static ssize_t sftp_write(LIBSSH2_SFTP_HANDLE * handle, const char *buffer,
     rc = sftp_packet_require(sftp, SSH_FXP_STATUS,
                              sftp->write_request_id, &data, &data_len);
     if (rc == PACKET_EAGAIN) {
-        return PACKET_EAGAIN;
-    } else if (rc) {
-        libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
+        return rc;
+    }
+    else if (rc) {
+        libssh2_error(session, rc,
                       "Timeout waiting for status message", 0);
         sftp->write_state = libssh2_NB_state_idle;
-        return -1;
+        return rc;
     }
 
     sftp->write_state = libssh2_NB_state_idle;
@@ -1498,7 +1503,7 @@ static ssize_t sftp_write(LIBSSH2_SFTP_HANDLE * handle, const char *buffer,
                   0);
     sftp->last_errno = retcode;
 
-    return -1;
+    return LIBSSH2_ERROR_SFTP_PROTOCOL;
 }
 
 /* libssh2_sftp_write
@@ -1567,7 +1572,7 @@ static int sftp_fstat(LIBSSH2_SFTP_HANDLE *handle,
         rc = _libssh2_channel_write(channel, 0, (char *) sftp->fstat_packet,
                                     packet_len);
         if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return rc;
         } else if (packet_len != rc) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
                           (setstat ? "Unable to send FXP_FSETSTAT"
@@ -1587,7 +1592,7 @@ static int sftp_fstat(LIBSSH2_SFTP_HANDLE *handle,
                               sftp->fstat_request_id, &data,
                               &data_len);
     if (rc == PACKET_EAGAIN) {
-        return PACKET_EAGAIN;
+        return rc;
     } else if (rc) {
         libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
                       "Timeout waiting for status message", 0);
@@ -1654,7 +1659,10 @@ libssh2_sftp_seek64(LIBSSH2_SFTP_HANDLE * handle, libssh2_uint64_t offset)
 LIBSSH2_API size_t
 libssh2_sftp_tell(LIBSSH2_SFTP_HANDLE * handle)
 {
-    return handle->u.file.offset;
+    /* NOTE: this may very well truncate the size if it is larger than what
+       size_t can hold, so libssh2_sftp_tell64() is really the function you
+       should use */
+    return (size_t)(handle->u.file.offset);
 }
 
 /* libssh2_sftp_tell64
@@ -1710,7 +1718,7 @@ sftp_close_handle(LIBSSH2_SFTP_HANDLE *handle)
         rc = _libssh2_channel_write(channel, 0, (char *) handle->close_packet,
                                     packet_len);
         if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return rc;
         } else if (packet_len != rc) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
                           "Unable to send FXP_CLOSE command", 0);
@@ -1730,7 +1738,7 @@ sftp_close_handle(LIBSSH2_SFTP_HANDLE *handle)
                                  handle->close_request_id, &data,
                                  &data_len);
         if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return rc;
         } else if (rc) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
                           "Timeout waiting for status message", 0);
@@ -1752,12 +1760,8 @@ sftp_close_handle(LIBSSH2_SFTP_HANDLE *handle)
         return -1;
     }
 
-    if (handle == sftp->handles) {
-        sftp->handles = handle->next;
-    }
-    if (handle->next) {
-        handle->next->prev = NULL;
-    }
+    /* remove this handle from the parent's list */
+    _libssh2_list_remove(&handle->node);
 
     if ((handle->handle_type == LIBSSH2_SFTP_HANDLE_DIR)
         && handle->u.dir.names_left) {
@@ -1826,7 +1830,7 @@ static int sftp_unlink(LIBSSH2_SFTP *sftp, const char *filename,
         rc = _libssh2_channel_write(channel, 0, (char *) sftp->unlink_packet,
                                     packet_len);
         if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return rc;
         } else if (packet_len != rc) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
                           "Unable to send FXP_REMOVE command", 0);
@@ -1845,7 +1849,7 @@ static int sftp_unlink(LIBSSH2_SFTP *sftp, const char *filename,
                              sftp->unlink_request_id, &data,
                              &data_len);
     if (rc == PACKET_EAGAIN) {
-        return PACKET_EAGAIN;
+        return rc;
     }
     else if (rc) {
         libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
@@ -1894,7 +1898,8 @@ static int sftp_rename(LIBSSH2_SFTP *sftp, const char *source_filename,
 {
     LIBSSH2_CHANNEL *channel = sftp->channel;
     LIBSSH2_SESSION *session = channel->session;
-    unsigned long data_len, retcode;
+    unsigned long data_len;
+    int retcode;
     ssize_t packet_len =
         source_filename_len + dest_filename_len + 17 + (sftp->version >=
                                                         5 ? 4 : 0);
@@ -1948,7 +1953,7 @@ static int sftp_rename(LIBSSH2_SFTP *sftp, const char *source_filename,
         rc = _libssh2_channel_write(channel, 0, (char *) sftp->rename_packet,
                                     sftp->rename_s - sftp->rename_packet);
         if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return rc;
         } else if (packet_len != rc) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
                           "Unable to send FXP_RENAME command", 0);
@@ -1967,7 +1972,7 @@ static int sftp_rename(LIBSSH2_SFTP *sftp, const char *source_filename,
                              sftp->rename_request_id, &data,
                              &data_len);
     if (rc == PACKET_EAGAIN) {
-        return PACKET_EAGAIN;
+        return rc;
     } else if (rc) {
         libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
                       "Timeout waiting for status message", 0);
@@ -2079,11 +2084,11 @@ static int sftp_mkdir(LIBSSH2_SFTP *sftp, const char *path,
         rc = _libssh2_channel_write(channel, 0, (char *) packet, packet_len);
         if (rc == PACKET_EAGAIN) {
             sftp->mkdir_packet = packet;
-            return PACKET_EAGAIN;
+            return rc;
         }
         if (packet_len != rc) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
-                          "Unable to send FXP_READ command", 0);
+                          "_libssh2_channel_write() failed", 0);
             LIBSSH2_FREE(session, packet);
             sftp->mkdir_state = libssh2_NB_state_idle;
             return -1;
@@ -2096,7 +2101,7 @@ static int sftp_mkdir(LIBSSH2_SFTP *sftp, const char *path,
     rc = sftp_packet_require(sftp, SSH_FXP_STATUS, sftp->mkdir_request_id,
                              &data, &data_len);
     if (rc == PACKET_EAGAIN) {
-        return PACKET_EAGAIN;
+        return rc;
     } else if (rc) {
         libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
                       "Timeout waiting for status message", 0);
@@ -2177,7 +2182,7 @@ static int sftp_rmdir(LIBSSH2_SFTP *sftp, const char *path,
         rc = _libssh2_channel_write(channel, 0, (char *) sftp->rmdir_packet,
                                     packet_len);
         if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return rc;
         } else if (packet_len != rc) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
                           "Unable to send FXP_RMDIR command", 0);
@@ -2195,7 +2200,7 @@ static int sftp_rmdir(LIBSSH2_SFTP *sftp, const char *path,
     rc = sftp_packet_require(sftp, SSH_FXP_STATUS,
                              sftp->rmdir_request_id, &data, &data_len);
     if (rc == PACKET_EAGAIN) {
-        return PACKET_EAGAIN;
+        return rc;
     } else if (rc) {
         libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
                       "Timeout waiting for status message", 0);
@@ -2296,7 +2301,7 @@ static int sftp_stat(LIBSSH2_SFTP *sftp, const char *path,
         rc = _libssh2_channel_write(channel, 0, (char *) sftp->stat_packet,
                                     packet_len);
         if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return rc;
         } else if (packet_len != rc) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
                           "Unable to send STAT/LSTAT/SETSTAT command", 0);
@@ -2314,7 +2319,7 @@ static int sftp_stat(LIBSSH2_SFTP *sftp, const char *path,
     rc = sftp_packet_requirev(sftp, 2, stat_responses,
                               sftp->stat_request_id, &data, &data_len);
     if (rc == PACKET_EAGAIN) {
-        return PACKET_EAGAIN;
+        return rc;
     } else if (rc) {
         libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
                       "Timeout waiting for status message", 0);
@@ -2436,7 +2441,7 @@ static int sftp_symlink(LIBSSH2_SFTP *sftp, const char *path,
         rc = _libssh2_channel_write(channel, 0, (char *) sftp->symlink_packet,
                                     packet_len);
         if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
+            return rc;
         } else if (packet_len != rc) {
             libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
                           "Unable to send SYMLINK/READLINK command", 0);
@@ -2455,7 +2460,7 @@ static int sftp_symlink(LIBSSH2_SFTP *sftp, const char *path,
                               sftp->symlink_request_id, &data,
                               &data_len);
     if (rc == PACKET_EAGAIN) {
-        return PACKET_EAGAIN;
+        return rc;
     }
     else if (rc) {
         libssh2_error(session, LIBSSH2_ERROR_SOCKET_TIMEOUT,
