@@ -95,10 +95,10 @@ debugdump(LIBSSH2_SESSION * session,
 
 /* decrypt() decrypts 'len' bytes from 'source' to 'dest'.
  *
- * returns PACKET_NONE on success and PACKET_FAIL on failure
+ * returns 0 on success and negative on failure
  */
 
-static libssh2pack_t
+static int
 decrypt(LIBSSH2_SESSION * session, unsigned char *source,
         unsigned char *dest, int len)
 {
@@ -134,7 +134,7 @@ decrypt(LIBSSH2_SESSION * session, unsigned char *source,
  * fullpacket() gets called when a full packet has been received and properly
  * collected.
  */
-static libssh2pack_t
+static int
 fullpacket(LIBSSH2_SESSION * session, int encrypted /* 1 or 0 */ )
 {
     unsigned char macbuf[MAX_MACSIZE];
@@ -232,11 +232,8 @@ fullpacket(LIBSSH2_SESSION * session, int encrypted /* 1 or 0 */ )
         rc = _libssh2_packet_add(session, p->payload,
                                  session->fullpacket_payload_len,
                                  session->fullpacket_macstate);
-        if (rc == PACKET_EAGAIN) {
-            return PACKET_EAGAIN;
-        } else if (rc < 0) {
-            return PACKET_FAIL;
-        }
+        if (rc)
+            return rc;
     }
 
     session->fullpacket_state = libssh2_NB_state_idle;
@@ -248,22 +245,21 @@ fullpacket(LIBSSH2_SESSION * session, int encrypted /* 1 or 0 */ )
 /*
  * _libssh2_transport_read
  *
- * Collect a packet into the input brigade block only controls whether or not
- * to wait for a packet to start.
+ * Collect a packet into the input queue.
  *
- * Returns packet type added to input brigade (PACKET_NONE if nothing added),
- * or PACKET_FAIL on failure and PACKET_EAGAIN if it couldn't process a full
- * packet.
+ * Returns packet type added to input queue (0 if nothing added), or a
+ * negative error number.
  */
 
 /*
  * This function reads the binary stream as specified in chapter 6 of RFC4253
  * "The Secure Shell (SSH) Transport Layer Protocol"
+ *
+ * DOES NOT call libssh2_error() for ANY error case.
  */
-libssh2pack_t
-_libssh2_transport_read(LIBSSH2_SESSION * session)
+int _libssh2_transport_read(LIBSSH2_SESSION * session)
 {
-    libssh2pack_t rc;
+    int rc = LIBSSH2_ERROR_SOCKET_NONE;
     struct transportpacket *p = &session->packet;
     int remainbuf;
     int remainpack;
@@ -273,7 +269,8 @@ _libssh2_transport_read(LIBSSH2_SESSION * session)
     int blocksize;
     int encrypted = 1;
 
-    int status;
+    /* default clear the bit */
+    session->socket_block_directions &= ~LIBSSH2_SESSION_BLOCK_INBOUND;
 
     /*
      * All channels, systems, subsystems, etc eventually make it down here
@@ -294,17 +291,10 @@ _libssh2_transport_read(LIBSSH2_SESSION * session)
          * is done!
          */
         _libssh2_debug(session, LIBSSH2_DBG_TRANS, "Redirecting into the"
-            " key re-exchange");
-        status = libssh2_kex_exchange(session, 1, &session->startup_key_state);
-        if (status == PACKET_EAGAIN) {
-            libssh2_error(session, LIBSSH2_ERROR_EAGAIN,
-                "Would block exchanging encryption keys", 0);
-            return PACKET_EAGAIN;
-      } else if (status) {
-          libssh2_error(session, LIBSSH2_ERROR_KEX_FAILURE,
-                      "Unable to exchange encryption keys",0);
-          return LIBSSH2_ERROR_KEX_FAILURE;
-      }
+                       " key re-exchange");
+        rc = libssh2_kex_exchange(session, 1, &session->startup_key_state);
+        if (rc)
+            return rc;
     }
 
     /*
@@ -363,18 +353,19 @@ _libssh2_transport_read(LIBSSH2_SESSION * session)
             /* now read a big chunk from the network into the temp buffer */
             nread =
                 _libssh2_recv(session->socket_fd, &p->buf[remainbuf],
-                     PACKETBUFSIZE - remainbuf,
-                     LIBSSH2_SOCKET_RECV_FLAGS(session));
+                              PACKETBUFSIZE - remainbuf,
+                              LIBSSH2_SOCKET_RECV_FLAGS(session));
             if (nread <= 0) {
                 /* check if this is due to EAGAIN and return the special
                    return code if so, error out normally otherwise */
                 if ((nread < 0) && (errno == EAGAIN)) {
-                    session->socket_block_directions =
+                    session->socket_block_directions |=
                         LIBSSH2_SESSION_BLOCK_INBOUND;
                     return PACKET_EAGAIN;
                 }
                 return PACKET_FAIL;
             }
+
             debugdump(session, "libssh2_transport_read() raw",
                       &p->buf[remainbuf], nread);
             /* advance write pointer */
@@ -397,6 +388,8 @@ _libssh2_transport_read(LIBSSH2_SESSION * session)
                    check is only done for the initial block since once we have
                    got the start of a block we can in fact deal with fractions
                 */
+                session->socket_block_directions |=
+                    LIBSSH2_SESSION_BLOCK_INBOUND;
                 return PACKET_EAGAIN;
             }
 
@@ -421,7 +414,12 @@ _libssh2_transport_read(LIBSSH2_SESSION * session)
              * and we can extract packet and padding length from it
              */
             p->packet_length = _libssh2_ntohu32(block);
+            if (p->packet_length < 1)
+                return PACKET_FAIL;
+
             p->padding_length = block[4];
+            if (p->padding_length < 0)
+                return PACKET_FAIL;
 
             /* total_num is the number of bytes following the initial
                (5 bytes) packet length and padding length fields */
@@ -562,7 +560,7 @@ _libssh2_transport_read(LIBSSH2_SESSION * session)
                     session->readPack_state = libssh2_NB_state_jump1;
                 }
 
-                return PACKET_EAGAIN;
+                return rc;
             }
 
             p->total_num = 0;   /* no packet buffer available */
@@ -574,7 +572,7 @@ _libssh2_transport_read(LIBSSH2_SESSION * session)
     return PACKET_FAIL;         /* we never reach this point */
 }
 
-static libssh2pack_t
+static int
 send_existing(LIBSSH2_SESSION * session, unsigned char *data,
               unsigned long data_len, ssize_t * ret)
 {
@@ -603,28 +601,32 @@ send_existing(LIBSSH2_SESSION * session, unsigned char *data,
     length = p->ototal_num - p->osent;
 
     rc = _libssh2_send(session->socket_fd, &p->outbuf[p->osent], length,
-              LIBSSH2_SOCKET_SEND_FLAGS(session));
+                       LIBSSH2_SOCKET_SEND_FLAGS(session));
+
+    if(rc > 0) {
+        debugdump(session, "libssh2_transport_write send()",
+                  &p->outbuf[p->osent], rc);
+    }
 
     if (rc == length) {
         /* the remainder of the package was sent */
         LIBSSH2_FREE(session, p->outbuf);
         p->outbuf = NULL;
         p->ototal_num = 0;
-    } else if (rc < 0) {
+    }
+    else if (rc < 0) {
         /* nothing was sent */
         if (errno != EAGAIN) {
             /* send failure! */
             return PACKET_FAIL;
         }
-        session->socket_block_directions = LIBSSH2_SESSION_BLOCK_OUTBOUND;
+        session->socket_block_directions |= LIBSSH2_SESSION_BLOCK_OUTBOUND;
         return PACKET_EAGAIN;
     }
 
-    debugdump(session, "libssh2_transport_write send()", &p->outbuf[p->osent],
-              length);
-    p->osent += length;         /* we sent away this much data */
+    p->osent += rc;         /* we sent away this much data */
 
-    return PACKET_NONE;
+    return p->osent < data_len ? PACKET_EAGAIN : PACKET_NONE;
 }
 
 /*
@@ -641,6 +643,8 @@ send_existing(LIBSSH2_SESSION * session, unsigned char *data,
  * NOTE: this function does not verify that 'data_len' is less than ~35000
  * which is what all implementations should support at least as packet size.
  * (RFC4253 section 6.1)
+ *
+ * This function DOES not call libssh2_error() on any errors.
  */
 int
 _libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
@@ -661,12 +665,15 @@ _libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
     int encrypted;
     int i;
     ssize_t ret;
-    libssh2pack_t rc;
+    int rc;
     unsigned char *orgdata = data;
     unsigned long orgdata_len = data_len;
 
     debugdump(session, "libssh2_transport_write plain", data, data_len);
 
+    /* default clear the bit */
+    session->socket_block_directions &= ~LIBSSH2_SESSION_BLOCK_OUTBOUND;
+
     /* FIRST, check if we have a pending write to complete */
     rc = send_existing(session, data, data_len, &ret);
     if (rc || ret) {
@@ -768,7 +775,7 @@ _libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
     session->local.seqno++;
 
     ret = _libssh2_send(session->socket_fd, p->outbuf, total_length,
-               LIBSSH2_SOCKET_SEND_FLAGS(session));
+                        LIBSSH2_SOCKET_SEND_FLAGS(session));
 
     if (ret != -1) {
         debugdump(session, "libssh2_transport_write send()", p->outbuf, ret);
@@ -776,7 +783,7 @@ _libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
     if (ret != total_length) {
         if ((ret > 0) || ((ret == -1) && (errno == EAGAIN))) {
             /* the whole packet could not be sent, save the rest */
-            session->socket_block_directions = LIBSSH2_SESSION_BLOCK_OUTBOUND;
+            session->socket_block_directions |= LIBSSH2_SESSION_BLOCK_OUTBOUND;
             p->odata = orgdata;
             p->olen = orgdata_len;
             p->osent = (ret == -1) ? 0 : ret;
@@ -794,6 +801,3 @@ _libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
 
     return PACKET_NONE;         /* all is good */
 }
-
-
-
